@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- The Competitor entity
-# :Creato:    gio 27 nov 2008 13:51:08 CET
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- The Competitor entity
+# :Created:   gio 27 nov 2008 13:51:08 CET
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2016, 2018, 2019, 2020 Lele Gaifax
 #
 
 from decimal import Decimal
@@ -12,7 +13,7 @@ from sqlalchemy import Column, ForeignKey, Index, Sequence
 from sqlalchemy.ext.declarative import declared_attr
 from sqlalchemy.orm import relationship
 
-from ..i18n import translatable_string as N_, gettext as _
+from ..i18n import gettext, translatable_string as _
 from . import Base
 from .domains import boolean_t, intid_t, prize_t, smallint_t
 
@@ -55,91 +56,100 @@ class Competitor(Base):
         intid_t, Sequence('gen_idcompetitor', optional=True),
         primary_key=True,
         nullable=False,
-        info=dict(label=N_('Competitor ID'),
-                  hint=N_('Unique ID of the competitor.')))
+        info=dict(label=_('Competitor ID'),
+                  hint=_('Unique ID of the competitor.')))
     """Primary key."""
 
     idtourney = Column(
-        intid_t, ForeignKey('tourneys.idtourney'),
+        intid_t, ForeignKey('tourneys.idtourney', name='fk_competitor_tourney'),
         nullable=False,
-        info=dict(label=N_('Tourney ID'),
-                  hint=N_('ID of the tourney the competitor belongs to.')))
+        info=dict(label=_('Tourney ID'),
+                  hint=_('ID of the tourney the competitor belongs to.')))
     """Subscribed :py:class:`tourney <.Tourney>`'s ID."""
 
     idplayer1 = Column(
-        intid_t, ForeignKey('players.idplayer'),
+        intid_t, ForeignKey('players.idplayer', name='fk_competitor_player1'),
         nullable=False,
-        info=dict(label=N_('Player ID'),
-                  hint=N_('ID of the player.')))
+        info=dict(label=_('Player ID'),
+                  hint=_('ID of the player.')))
     """First :py:class:`player <.Player>`'s ID."""
 
     idplayer2 = Column(
-        intid_t, ForeignKey('players.idplayer'),
-        info=dict(label=N_('2nd player ID'),
-                  hint=N_('ID of the second player.')))
+        intid_t, ForeignKey('players.idplayer', name='fk_competitor_player2'),
+        info=dict(label=_('2nd player ID'),
+                  hint=_('ID of the second player.')))
     """Second :py:class:`player <.Player>`'s ID."""
 
     idplayer3 = Column(
-        intid_t, ForeignKey('players.idplayer'),
-        info=dict(label=N_('3rd player ID'),
-                  hint=N_('ID of the third player.')))
+        intid_t, ForeignKey('players.idplayer', name='fk_competitor_player3'),
+        info=dict(label=_('3rd player ID'),
+                  hint=_('ID of the third player.')))
     """Third :py:class:`player <.Player>`'s ID."""
 
     idplayer4 = Column(
-        intid_t, ForeignKey('players.idplayer'),
-        info=dict(label=N_('4th player ID'),
-                  hint=N_('ID of the fourth player.')))
+        intid_t, ForeignKey('players.idplayer', name='fk_competitor_player4'),
+        info=dict(label=_('4th player ID'),
+                  hint=_('ID of the fourth player.')))
     """Fourth :py:class:`player <.Player>`'s ID."""
 
     points = Column(
         smallint_t,
         nullable=False,
         default=0,
-        info=dict(label=N_('Points'),
-                  hint=N_('Points of the competitor.')))
+        info=dict(label=_('Points'),
+                  hint=_('Points of the competitor.')))
     """Points (number of wins * 2 + number of draws)."""
 
     bucholz = Column(
         smallint_t,
         nullable=False,
         default=0,
-        info=dict(label=N_('Bucholz'),
-                  hint=N_('Weight of the opponents.')))
+        info=dict(label=_('Bucholz'),
+                  hint=_('Weight of the opponents.')))
     """*Weight* of the opponents (sum of opponents' points and netscore)."""
 
     netscore = Column(
         smallint_t,
         nullable=False,
         default=0,
-        info=dict(label=N_('Net score'),
-                  hint=N_('Net score of all games.')))
+        info=dict(label=_('Net score'),
+                  hint=_('Net score of all games.')))
     """Net score (sum of carrommen difference in each match)."""
 
     totscore = Column(
         smallint_t,
         nullable=False,
         default=0,
-        info=dict(label=N_('Total score'),
-                  hint=N_('Total score of all games.')))
+        info=dict(label=_('Total score'),
+                  hint=_('Total score of all games.')))
     """Total score (sum of carrommen in each match)."""
 
     prize = Column(
         prize_t,
         nullable=False,
         default=0.0,
-        info=dict(label=N_('Final prize'),
-                  hint=N_('Final prize assigned at end of tourney.')))
+        info=dict(label=_('Final prize'),
+                  hint=_('Final prize assigned at end of tourney.')))
     """Final prize."""
 
     retired = Column(
         boolean_t,
         nullable=False,
         default=False,
-        info=dict(label=N_('Retired'),
-                  hint=N_('Whether this competitor will play'
-                          ' further matches.')))
+        info=dict(label=_('Withdrawn'),
+                  hint=_('Whether this competitor will play further matches.')))
     """A competitor may stop playing in the middle of the tourney."""
 
+    position = Column(
+        smallint_t,
+        nullable=False,
+        default=0,
+        info=dict(label=_('Position'),
+                  hint=_('Relative position used to order competitors when generating the'
+                         ' first round, overriding their current rate.')))
+    """The relative position of this competitor, used to generate the first round in knockout
+    tournaments, overriding their current rate."""
+
     ## Relations
 
     player1 = relationship('Player',
@@ -162,25 +172,31 @@ class Competitor(Base):
                            lazy='joined')
     """Fourth :py:class:`player <.Player>`."""
 
-    def __repr__(self): # pragma: no cover
-        r = super(Competitor, self).__repr__()
+    def __repr__(self):  # pragma: no cover
+        r = super().__repr__()
         r = r[:-1] + ' of t%s: p=%s b=%s ns=%s>' % (repr(self.tourney)[2:-1],
                                                     self.points,
                                                     self.bucholz,
                                                     self.netscore)
+        if self.position:
+            r = r[:-1] + ' #%d>' % self.position
         return r
 
-    def caption(self, html=None, localized=True, player_caption=None, css_class=None):
+    def caption(self, html=None, localized=True, player_caption=None, css_class=None,
+                nationality=False):
         "A description of the competitor, made up with the name of each player."
 
         from .utils import njoin
 
+        ignore_privacy = not self.tourney.prized if self.tourney else False
+
         if player_caption is None:
-            player_caption = lambda player, h, l, cc: player.caption(html=h, localized=l,
-                                                                css_class=cc)
+            def player_caption(player, h, l, cc):
+                return player.caption(html=h, localized=l, css_class=cc,
+                                      ignore_privacy=ignore_privacy)
 
         if self.player1 is None:
-            return _('Player NOT assigned yet!', just_subst=not localized)
+            return gettext('Player NOT assigned yet!', just_subst=not localized)
         else:
             captions = [player_caption(self.player1, html, localized, css_class)]
             if self.player2 is not None:
@@ -189,14 +205,21 @@ class Competitor(Base):
                 captions.append(player_caption(self.player3, html, localized, css_class))
             if self.player4 is not None:
                 captions.append(player_caption(self.player4, html, localized, css_class))
-            return njoin(captions, localized=localized)
+            caption = njoin(captions, localized=localized)
+            if nationality:
+                if (html is None or html):
+                    caption += ' <font size=6>(%s)</font>' % self.player1.nationality
+                else:
+                    caption += ' (%s)' % self.player1.nationality
+            return caption
 
     description = property(caption)
 
     @property
     def player1FullName(self):
         "Full name of the first player"
-        return self.idplayer1 and self.player1.description or None
+        if self.idplayer1:
+            return self.player1.caption(ignore_privacy=not self.tourney.prized)
 
     @property
     def player1Nationality(self):
@@ -223,7 +246,8 @@ class Competitor(Base):
     @property
     def player2FullName(self):
         "Full name of the second player"
-        return self.idplayer2 and self.player2.description or None
+        if self.idplayer2:
+            return self.player2.caption(ignore_privacy=not self.tourney.prized)
 
     @property
     def player2Nationality(self):
@@ -233,7 +257,8 @@ class Competitor(Base):
     @property
     def player3FullName(self):
         "Full name of the third player"
-        return self.idplayer3 and self.player3.description or None
+        if self.idplayer3:
+            return self.player3.caption(ignore_privacy=not self.tourney.prized)
 
     @property
     def player3Nationality(self):
@@ -243,7 +268,8 @@ class Competitor(Base):
     @property
     def player4FullName(self):
         "Full name of the fourth player"
-        return self.idplayer4 and self.player4.description or None
+        if self.idplayer4:
+            return self.player4.caption(ignore_privacy=not self.tourney.prized)
 
     @property
     def player4Nationality(self):
@@ -298,9 +324,27 @@ class Competitor(Base):
         ranking = self.tourney.ranking
         return ranking.index(self) + 1
 
+    def getOpponentsPreceedingTurns(self, turn):
+        "List of competitors ID who played against this competitor."
+
+        cid = self.idcompetitor
+        oppids = []
+
+        for match in self.tourney.matches:
+            if match.turn >= turn:
+                break
+
+            if match.idcompetitor1 == cid and match.idcompetitor2:
+                oppids.append(match.idcompetitor2)
+            elif match.idcompetitor2 == cid:
+                oppids.append(match.idcompetitor1)
+
+        return oppids
+
     def update(self, data, missing_only=False):
-        if 'prize' in data and isinstance(data['prize'], str):
-            data['prize'] = Decimal(data['prize'])
+        value = data.get('prize', None)
+        if isinstance(value, str):
+            data['prize'] = Decimal(value)
         return super().update(data, missing_only)
 
     def serialize(self, serializer):
@@ -317,10 +361,10 @@ class Competitor(Base):
             pers.append(serializer.addPlayer(self.player1))
         if self.idplayer2:
             pers.append(serializer.addPlayer(self.player2))
-        if self.idplayer3:
+        if self.idplayer3:  # pragma: nocover
             pers.append(serializer.addPlayer(self.player3))
-        if self.idplayer4:
-            pers.append(serializer.addPlayer(self.player4))
+            if self.idplayer4:  # pragma: nocover
+                pers.append(serializer.addPlayer(self.player4))
         simple['points'] = self.points
         simple['netscore'] = self.netscore
         simple['totscore'] = self.totscore
