@@ -3,7 +3,7 @@
 # :Created:   gio 27 nov 2008 13:52:02 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020, 2021 Lele Gaifax
+# :Copyright: © 2008–2010, 2013–2015, 2016, 2018, 2020, 2021, 2023 Lele Gaifax
 #
 
 import logging
@@ -289,3 +289,29 @@ class Match(Base):
         s = Signer(settings['sol.signer_secret_key'])
         signed_match = s.sign('%d-%d' % (self.idmatch, cnum)).decode('ascii')
         return request.route_url('training_match_form', match=signed_match)
+
+    def compute_partial_scores(self):
+        "Enrich played boards with partial scores."
+
+        total1 = total2 = self.partial_score1 = self.partial_score2 = 0
+        if not self.boards:
+            return
+        for board in self.boards:
+            score1 = board.coins1 or 0
+            score2 = board.coins2 or 0
+            if score1 == score2 == 0:
+                if board.queen == '1':
+                    score1 += 3 if total1 < 22 else 1
+                elif board.queen == '2':
+                    score2 += 3 if total2 < 22 else 1
+            else:
+                if board.queen == '1' and score1 > score2 and total1 < 22:
+                    score1 += 3
+                elif board.queen == '2' and score1 < score2 and total2 < 22:
+                    score2 += 3
+            board.score1 = score1
+            board.score2 = score2
+            total1 += score1
+            total2 += score2
+        self.partial_score1 = total1
+        self.partial_score2 = total2
