@@ -3,7 +3,7 @@
 # :Created:   gio 27 nov 2008 13:52:39 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013-2020, 2021 Lele Gaifax
+# :Copyright: © 2008, 2009, 2010, 2013-2020, 2021, 2022 Lele Gaifax
 #
 
 from base64 import b32encode
@@ -70,6 +70,18 @@ Thanks for playing, happy carromming and stay safe!
 """)
 
 
+TRAINING_PHANTOM_MESSAGE = _("""\
+Hello {firstname},
+
+you are participating in the long-distance “{tourney}” Carrom event.
+
+In the next round you are going to play against the Phantom, which means
+you have a free round.
+
+Thanks for playing, happy carromming and stay safe!
+""")
+
+
 TRAINING_URLS_MESSAGE = _("""\
 Hello {firstname},
 
@@ -923,7 +935,8 @@ class Player(GloballyUnique, Base):
         :param request: the web request
         :param match: the :py:class:`.Match` instance
         :param int cnum: the concurrent number
-        :param opponent: the opponent :py:class:`.Player` instance
+        :param opponent: the opponent :py:class:`.Player` instance, or ``None`` for the
+                         *Phantom*
         """
 
         language = self.language or request.registry.settings['default_locale_name']
@@ -934,22 +947,25 @@ class Player(GloballyUnique, Base):
         t = loc.translate
         if opponent is not None:
             oname = t(opponent.caption(html=False, ignore_privacy=True))
-        else:  # pragma: nocover
-            oname = t(_("Phantom"))
-        nboards = match.tourney.championship.trainingboards
-        if match.tourney.socialurl:
-            details = "\n" + t(_('Do not forget to upload your video on ${socialurl}.',
-                                 mapping=dict(socialurl=match.tourney.socialurl))) + "\n"
+            nboards = match.tourney.championship.trainingboards
+            if match.tourney.socialurl:
+                details = "\n" + t(_('Do not forget to upload your video on ${socialurl}.',
+                                     mapping=dict(socialurl=match.tourney.socialurl))) + "\n"
+            else:
+                details = ""
+            url = match.getEditCompetitorURL(request, cnum)
+            body = t(TRAINING_URL_MESSAGE).format(firstname=self.firstname,
+                                                  tourney=match.tourney.description,
+                                                  opponent=oname, nboards=nboards,
+                                                  url=url, details=details)
         else:
-            details = ""
-        url = match.getEditCompetitorURL(request, cnum)
-        body = t(TRAINING_URL_MESSAGE).format(firstname=self.firstname,
-                                              tourney=match.tourney.description,
-                                              opponent=oname, nboards=nboards,
-                                              url=url, details=details)
-        message = Message(subject=t(_('Your next SoLitude-Carrom match')),
-                          recipients=[self.email],
-                          body=body)
+            oname = t(_("Phantom"))
+            body = t(TRAINING_PHANTOM_MESSAGE).format(firstname=self.firstname,
+                                                      tourney=match.tourney.description,
+                                                      opponent=oname)
+            url = None
+        subject = t(_('Your next SoLitude-Carrom match'))
+        message = Message(subject=subject, recipients=[self.email], body=body)
         request.mailer.send(message)
         logger.debug('Sent self-edit link to %r: %s', self, url)
 
