@@ -3,7 +3,7 @@
 # :Created:   gio 05 dic 2013 09:05:58 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
+# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2020, 2022 Lele Gaifax
 #
 
 from datetime import date
@@ -353,6 +353,19 @@ class Rating(GloballyUnique, Base):
                     and competitor.totscore == 0
                     and competitor.netscore % competitor.tourney.phantomscore == 0))
 
+    def shouldConsiderTourney(self, tourney):
+        """Determine whether the given tourney should be considered.
+
+        :param tourney: a Tourney instance
+
+        Only singles using the Swiss System should be considered. Also, "online" tournaments
+        are excluded.
+        """
+
+        return ((tourney.system == 'swiss'
+                 and tourney.championship.playersperteam == 1
+                 and not tourney.championship.trainingboards))
+
     def _computeGlickoOutcomes(self, score1, score2):
         # Standard Glicko, best suited to Chess matches
 
@@ -472,15 +485,7 @@ class Rating(GloballyUnique, Base):
             if mindate is not None and tourney.date < mindate:
                 continue
 
-            if tourney.championship.playersperteam > 1:
-                logger.warning('Cannot update %r for %r: only singles supported, sorry!',
-                               self, tourney)
-                continue
-
-            if not tourney.prized:
-                continue
-
-            if tourney.system == 'knockout':
+            if not tourney.prized or not self.shouldConsiderTourney(tourney):
                 continue
 
             logger.debug('Considering tourney %s', tourney)
