@@ -3,15 +3,15 @@
 # :Created:   gio 27 nov 2008 13:54:14 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021 Lele Gaifax
+# :Copyright: © 2008-2010, 2013-2016, 2018-2023 Lele Gaifax
 #
 
-from collections import namedtuple
+from collections import Counter, defaultdict, namedtuple
 from fractions import Fraction
 from itertools import chain, combinations
 import logging
 from math import log2
-from operator import itemgetter
+from operator import attrgetter, itemgetter
 
 from itsdangerous import Signer
 from sqlalchemy import Column, ForeignKey, Index, Sequence, func, exists, or_, select
@@ -46,9 +46,9 @@ class RankingStats:
     """
 
     __slots__ = ('points', 'netscore', 'totscore', 'bucholz', 'real_points',
-                 'played_matches', 'virtual_points')
+                 'played_matches', 'virtual_points', 'retired')
 
-    def __init__(self):
+    def __init__(self, retired):
         self.points = 0
         "Overall points."
 
@@ -70,6 +70,9 @@ class RankingStats:
         self.virtual_points = 0
         "Estimated further points, after retirement."
 
+        self.retired = retired
+        "Whether it is a retired competitor."
+
     def won(self, netscore, against_phantom):
         "Update stats after a winned match."
 
@@ -461,8 +464,6 @@ class Tourney(GloballyUnique, Base):
     def _computeFinalWins(self):
         """Compute the number of matches won by each competitor in the finals"""
 
-        from collections import Counter
-
         finalmatches = [m for m in self.matches if m.final]
         nfinalturns = len(set(m.turn for m in finalmatches))
 
@@ -537,11 +538,12 @@ class Tourney(GloballyUnique, Base):
         """
 
         # Start from scratch, assigning zero to all competitors
-        ranking = {comp: RankingStats() for comp in self.competitors}
+        ranking = {comp: RankingStats(comp.retired) for comp in self.competitors}
 
         is_aaat = self.isAllAgainstAllTraining
 
         # First of all, sum up points and netscore
+        highest_scored_turn = 0
         for match in self.matches:
             if (turn is not None and match.turn > turn) or match.final:
                 break
@@ -549,6 +551,8 @@ class Tourney(GloballyUnique, Base):
             if is_aaat and not match.isScored:
                 continue
 
+            highest_scored_turn = match.turn
+
             winner, loser, netscore = match.results()
             if netscore == 0:
                 ranking[winner].drawn()
@@ -566,9 +570,9 @@ class Tourney(GloballyUnique, Base):
             else:
                 assert self.retirements == 'trend70'
                 factor = Fraction(70, 100)
-            current_turn = self.currentturn
-            for c, r in ranking.items():
-                if r.played_matches < current_turn:
+            current_turn = highest_scored_turn if turn is None else turn
+            for r in ranking.values():
+                if r.retired:
                     if r.real_points != 0:
                         average = Fraction(r.real_points, r.played_matches) * factor
                         adjustment = average * (current_turn - r.played_matches)
@@ -577,7 +581,7 @@ class Tourney(GloballyUnique, Base):
                     r.virtual_points = int(adjustment)
 
         # Add phantom
-        ranking[None] = RankingStats()
+        ranking[None] = RankingStats(False)
 
         for match in self.matches:
             if (turn is not None and match.turn > turn) or match.final:
@@ -799,13 +803,26 @@ class Tourney(GloballyUnique, Base):
             """
 
             same_points = self.countSamePointsAsPivot()
-            if same_points > 3:
+            all_players = len(self.competitors)
+            if same_points > 2:
+                # If the group cardinality is odd, try to include the first player of
+                # the next group
+                if same_points < all_players and same_points % 2:
+                    # Considering the presence of the phantom, don't to that if we are
+                    # building the very first round, ie when all players have 0 points
+                    if None not in self.competitors or same_points != (all_players - 1):
+                        if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
+                            border_c = self.competitors[same_points].caption(False, False)
+                            logger.debug("Considering competitor “%s” as belonging to"
+                                         " the group with %d points",
+                                         border_c, self.pivot.points)
+                        same_points += 1
                 middle = same_points // 2
                 positions = chain(range(middle, same_points),
                                   range(0, middle),
-                                  range(same_points, len(self.competitors)))
+                                  range(same_points, all_players))
             else:
-                positions = range(len(self.competitors))
+                positions = range(all_players)
             return positions
 
     class StaggeredVisitor(DazedVisitor):
@@ -820,18 +837,29 @@ class Tourney(GloballyUnique, Base):
 
         def computeVisitOrder(self):
             same_points = self.countSamePointsAsPivot()
-            if same_points > 3:
+            all_players = len(self.competitors)
+            if same_points > 2:
+                if same_points < all_players and same_points % 2:
+                    # Considering the presence of the phantom, don't to that if we are
+                    # building the very first round, ie when all players have 0 points
+                    if None not in self.competitors or same_points != (all_players - 1):
+                        if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
+                            border_c = self.competitors[same_points].caption(False, False)
+                            logger.debug("Considering competitor “%s” as belonging to"
+                                         " the group with %d points",
+                                         border_c, self.pivot.points)
+                        same_points += 1
                 if same_points >= 50:
                     positions = chain(range(25, same_points),
                                       range(0, 25),
-                                      range(same_points, len(self.competitors)))
+                                      range(same_points, all_players))
                 else:
                     middle = same_points // 2
                     positions = chain(range(middle, same_points),
                                       range(0, middle),
-                                      range(same_points, len(self.competitors)))
+                                      range(same_points, all_players))
             else:
-                positions = range(len(self.competitors))
+                positions = range(all_players)
             return positions
 
     def _combine(self, competitors, done, _level=0):
@@ -844,25 +872,33 @@ class Tourney(GloballyUnique, Base):
         if logger.isEnabledFor(logging.DEBUG):  # pragma: nocover
             from pprint import pformat
 
+            ranking = self.ranking
+
             def debug(msg, *args):
                 logger.debug("%sL%02d "+msg, "="*_level, _level, *args)
 
             def C(c):
-                return c.caption(False, False) if c else "Phantom"
+                if c:
+                    position = f"{ranking.index(c) + 1}. "
+                    return f"{position}{c.caption(False, False)} ({c.points}p)"
+                else:
+                    return "Phantom"
 
-            if _level == 0:
+            if _level == 0 and done:
                 done_matches = {}
                 for c1, c2 in done:
                     if c1:
-                        matches = done_matches.setdefault(C(c1), [])
-                        if C(c2) not in matches:
-                            matches.append(C(c2))
-                            matches = done_matches.setdefault(C(c2), [])
-                            matches.append(C(c1))
-                debug("Done matches:\n%s", pformat(done_matches))
+                        matches = done_matches.setdefault(c1, [])
+                        if c2 not in matches:
+                            matches.append(c2)
+                            matches = done_matches.setdefault(c2, [])
+                            matches.append(c1)
+                logger.debug("Done matches:\n%s",
+                             '\n'.join(f"  {C(c)}: {', '.join(C(o) for o in done_matches[c])}"
+                                       for c in ranking))
 
-            debug("Competitors with points: %s", [(i, C(c), c.points if c is not None else 0)
-                                                  for i, c in enumerate(competitors, 1)])
+            debug("Competitors to be paired:\n%s",
+                  '\n'.join(f"  {C(c)}" for c in competitors))
         else:
             debug = None
 
@@ -879,16 +915,16 @@ class Tourney(GloballyUnique, Base):
 
         c1 = competitors[0]
         if debug:  # pragma: nocover
-            remainingc = [(n, C(c))
-                          for n, c in enumerate(visitor(self, c1, competitors, done), 1)]
+            remainingc = tuple(visitor(self, c1, competitors, done))
             if remainingc:
-                debug('Looking for a competitor for %s within %s', C(c1), remainingc)
+                debug('Looking for a competitor for %s within\n%s', C(c1),
+                      '\n'.join(f"  {C(c)}" for c in remainingc))
             else:
                 debug('Backtracking: no possible competitors for %s', C(c1))
 
         for n, c2 in enumerate(visitor(self, c1, competitors, done), 1):
             if debug:  # pragma: nocover
-                debug("Tentative %d: trying %s,%s", n, C(c1), C(c2))
+                debug("Tentative %d: trying %s vs %s", n, C(c1), C(c2))
             if len(competitors) > 2:
                 remainings = self._combine(
                     [c for c in competitors if c is not c1 and c is not c2],
@@ -897,47 +933,36 @@ class Tourney(GloballyUnique, Base):
                     newturn = [(c1, c2)] + remainings
                     if debug:  # pragma: nocover
                         if _level == 0:
-                            debug("OK => %s", [(C(_c1), C(_c2)) for _c1, _c2 in newturn])
+                            debug("OK =>\n%s",
+                                  '\n'.join(f"  {C(_c1)} vs {C(_c2)}" for _c1, _c2 in newturn))
                         else:
                             debug("OK")
                     return newturn
             else:
                 if debug:  # pragma: nocover
-                    debug("OK => %s,%s", C(c1), C(c2))
+                    debug("OK => %s vs %s", C(c1), C(c2))
                 return [(c1, c2)]
 
-    def _assignBoards(self, matches, comp_boards, available_boards=None):
+    def _assignBoards(self, matches, comp_boards):
         """Assign a table to each match, possibly one not already used."""
 
-        if len(matches) == 0:
-            return True
+        available_boards = range(1, len(matches)+1)
+        for match in matches:
+            boards_usage = comp_boards[match.competitor1]
+            # Consider the second competitor only when there are more than
+            # 6 boards/12 competitors: under that threshold it's better to
+            # try making the first competitor to play an a different board
+            # each time. See issue #34.
+            if len(matches) > 6:
+                boards_usage = boards_usage + comp_boards[match.competitor2]
+            board = sorted(available_boards, key=boards_usage.__getitem__)[0]
 
-        if available_boards is None:
-            available_boards = range(1, len(matches)+1)
-
-        match = matches[0]
-        rem_matches = matches[1:]
-
-        # Try to assign a table not used by both competitors
-        for board in available_boards:
-            rem_boards = [b for b in available_boards if b != board]
-            if ((board not in comp_boards[match.competitor1] and
-                 board not in comp_boards[match.competitor2] and
-                 self._assignBoards(rem_matches, comp_boards, rem_boards))):
-                match.board = board
-                return True
-
-        # No solution, pick the first table
-        board = available_boards[0]
-        match.board = board
-        return self._assignBoards(rem_matches, comp_boards,
-                                  available_boards[1:])
+            available_boards = [b for b in available_boards if b != board]
+            match.board = board
 
     def _makeNextTurn(self):
         """Build the next round of the game."""
 
-        from collections import defaultdict
-        from operator import attrgetter
         from . import Match
 
         if self.system == 'knockout' and self.couplings == 'seeds':
@@ -959,9 +984,9 @@ class Tourney(GloballyUnique, Base):
 
         done = set()
         last = 0
-        # A dictionary with a set of used boards for each competitor,
+        # A dictionary with a counter of used boards for each competitor,
         # used later by _assignBoards()
-        comp_boards = defaultdict(set)
+        comp_boards = defaultdict(Counter)
         for m in self.matches:
             if m.turn > last:
                 last = m.turn
@@ -969,8 +994,8 @@ class Tourney(GloballyUnique, Base):
             c2 = m.competitor2
             done.add((c1, c2))
             done.add((c2, c1))
-            comp_boards[c1].add(m.board)
-            comp_boards[c2].add(m.board)
+            comp_boards[c1][m.board] += 1
+            comp_boards[c2][m.board] += 1
 
         newturn = last+1
         if self.system == 'swiss' or newturn == 1:
@@ -1018,12 +1043,6 @@ class Tourney(GloballyUnique, Base):
                               score1=0, score2=0)
                     matches.append(m)
 
-                # Take care of newly registered competitors
-                if c1 not in comp_boards:
-                    comp_boards[c1] = set()
-                if c2 not in comp_boards:
-                    comp_boards[c2] = set()
-
             if self.system == 'swiss':
                 self._assignBoards(matches, comp_boards)
             else:
@@ -1147,7 +1166,7 @@ class Tourney(GloballyUnique, Base):
         self.prized = True
         self.modified = func.now()
 
-        if self.rating is not None and self.system != 'knockout':
+        if self.rating is not None and self.rating.shouldConsiderTourney(self):
             self.rating.recompute(self.date)
 
         logger.info('Assigned final prizes for %r', self)
@@ -1210,7 +1229,6 @@ class Tourney(GloballyUnique, Base):
             prize -= fraction
 
     def resetPrizes(self):
-
         """Reset assigned final points."""
 
         for c in self.competitors:
@@ -1218,7 +1236,7 @@ class Tourney(GloballyUnique, Base):
 
         self.prized = False
         self.modified = func.now()
-        if self.rating is not None:
+        if self.rating is not None and self.rating.shouldConsiderTourney(self):
             self.rating.recompute(self.date)
 
     def replay(self, date, newidowner=None):
@@ -1391,21 +1409,23 @@ class Tourney(GloballyUnique, Base):
         if self.couplings == 'all':
             byp = {}
             for m in self.matches:
-                if m.competitor2 is None:  # pragma: nocover
-                    continue
                 pm = byp.setdefault(m.competitor1.player1, [])
-                pm.append((m, 1, m.competitor2.player1))
-                pm = byp.setdefault(m.competitor2.player1, [])
-                pm.append((m, 2, m.competitor1.player1))
+                pm.append((m, 1, m.competitor2.player1 if m.competitor2 is not None else None))
+                if m.competitor2 is not None:
+                    pm = byp.setdefault(m.competitor2.player1, [])
+                    pm.append((m, 2, m.competitor1.player1))
             for p in byp:
                 p.sendTrainingURLs(request, byp[p])
         else:
             for m in self.matches:
                 if m.turn == self.currentturn:
-                    if m.competitor2 is None:
-                        continue
-                    m.competitor1.player1.sendTrainingURL(request, m, 1, m.competitor2.player1)
-                    m.competitor2.player1.sendTrainingURL(request, m, 2, m.competitor1.player1)
+                    m.competitor1.player1.sendTrainingURL(request, m, 1,
+                                                          m.competitor2.player1
+                                                          if m.competitor2 is not None
+                                                          else None)
+                    if m.competitor2 is not None:
+                        m.competitor2.player1.sendTrainingURL(request, m, 2,
+                                                              m.competitor1.player1)
 
     def getEditBoardURL(self, request, boardno):
         settings = request.registry.settings
