@@ -3,7 +3,7 @@
 # :Created:   ven 31 ott 2008 10:32:29 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
+# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020, 2022 Lele Gaifax
 #
 
 """
@@ -120,7 +120,10 @@ cardname_style.leading = 10
 rank_width = 0.8*cm
 "The width of the `rank` columns"
 
-scores_width = 1*cm
+country_width = 0.9*cm
+"The width of the `country` columns"
+
+scores_width = 1.3*cm
 "The width of the `scores` columns"
 
 prizes_width = 2*cm
