@@ -1,54 +1,73 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL -- Configuration tool
-# :Creato:    gio 18 apr 2013 18:43:18 CEST
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL -- Configuration tool
+# :Created:   gio 18 apr 2013 18:43:18 CEST
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021 Lele Gaifax
 #
 
-from argparse import ArgumentParser
+from argparse import ArgumentParser, SUPPRESS
 from binascii import hexlify
 from datetime import date
-from os import urandom
-from os.path import abspath, dirname, exists, join
+from hashlib import md5
+from math import exp
+from os import makedirs, urandom
+from os.path import abspath, dirname, exists, join, realpath
+import re
+from shutil import copyfile
+import sys
+from time import time
 from urllib.request import urlopen
 
 from sqlalchemy import engine_from_config
+from sqlalchemy.orm import Session
 from sqlalchemy.orm.exc import MultipleResultsFound, NoResultFound
-import transaction
+
+from metapensiero.sqlalchemy.proxy.json import register_json_decoder_encoder
 
 from pyramid.paster import get_appsettings, setup_logging
 
-from sol.models import Base, Club, DBSession, Player, Rate, Rating, bio
+import sol
+from sol.models import Base, Club, Player, Rate, Rating, bio
 from sol.models.utils import normalize
+from sol.views import json_decode, json_encode
 
 
 def create_config(args):
     "Dump/update a configuration file suitable for production"
 
-    import sol
-
     if exists(args.config):
         print('The config file "%s" already exists!' % args.config)
         return update_config(args)
 
-    alembicdir = join(dirname(dirname(sol.__file__)), 'alembic')
+    alembicdir = args.alembic_dir or join(dirname(dirname(sol.__file__)), 'alembic')
     datadir = args.data_dir
     secret = hexlify(urandom(20)).decode('ascii')
+    signer_secret_key = hexlify(urandom(20)).decode('ascii')
     admin = args.admin
     password = args.password
-    if not password:
+    if not password:  # pragma: nocover
         password = input('Choose a password for the "%s" super user: ' % admin)
-    if not password:
+    if not password:  # pragma: nocover
         password = hexlify(urandom(5)).decode('ascii')
         random_password = True
     else:
         random_password = False
 
+    if not all(c.isalnum() for c in admin):
+        print("Invalid admin name, only letters and digits allowed!")
+        return 128
+
+    if len(password) < 8 or password.isdigit():
+        print("Invalid password, must be at least eight characters and not all digits!!")
+        return 128
+
     with open(join(dirname(__file__), "config.tpl"), encoding='utf-8') as f:
         configuration = f.read()
 
     with open(args.config, 'w', encoding='utf-8') as f:
-        f.write(configuration.format(secret=secret, admin=admin, password=password,
+        f.write(configuration.format(secret=secret, signer_secret_key=signer_secret_key,
+                                     admin=admin, password=password,
                                      alembicdir=alembicdir, datadir=datadir))
 
     print('The configuration file "%s" has been successfully created' % args.config)
@@ -59,26 +78,46 @@ def create_config(args):
 def update_config(args):
     "Update an existing configuration file"
 
-    import re
-    import sol
-
     if not exists(args.config):
         print('The config file "%s" does not exists!' % args.config)
-        return None
+        return 128
 
     with open(args.config, 'r', encoding='utf-8') as f:
         configuration = f.read()
 
+    alembicdir = args.alembic_dir or get_alembic_dir()
     substitutions = [(re.compile(r'script_location\s*=\s*(.+)\s*$'),
-                      join(dirname(dirname(sol.__file__)), 'alembic'),
+                      alembicdir,
                       'script_location = %s')]
-    if args.admin is not None:
+    admin = args.admin
+    if admin is not None:
+        if admin == 'ask':  # pragma: nocover
+            admin = input('Choose a name for the super user (defaults to "admin"): ' % admin)
+            admin = admin.strip()
+            if not admin:
+                admin = "admin"
+        else:
+            admin = admin.strip()
+
+        if not all(c.isalnum() for c in admin):
+            print("Invalid admin name, only letters and digits allowed!")
+            return 128
+
         substitutions.append((re.compile(r'sol\.admin\.user\s*=\s*(.+)\s*$'),
-                              args.admin,
+                              admin,
                               'sol.admin.user = %s'))
-    if args.password is not None:
+    password = args.password
+    if password is not None:
+        if password == 'ask':  # pragma: nocover
+            password = input('Choose a password for the super user: ')
+        password = password.strip()
+
+        if len(password) < 8 or password.isdigit():
+            print("Invalid password, must be at least eight characters and not all digits!!")
+            return 128
+
         substitutions.append((re.compile(r'sol\.admin\.password\s*=\s*(.+)\s*$'),
-                              args.password,
+                              password,
                               'sol.admin.password = %s'))
 
     updated = False
@@ -104,7 +143,7 @@ def update_config(args):
 def initialize_db(args):
     "Initialize the database structure"
 
-    if not exists(args.config):
+    if not exists(args.config):  # pragma: no cover
         print('The config file "%s" does not exist!' % args.config)
         return 128
 
@@ -112,12 +151,11 @@ def initialize_db(args):
     settings = get_appsettings(args.config)
     engine = engine_from_config(settings, 'sqlalchemy.')
 
-    if engine.has_table('tourneys'):
+    if engine.has_table('tourneys'):  # pragma: no cover
         print('Database "%s" already exist.'
               ' You may want to execute "upgrade-db" instead...' % engine.url)
         return None
 
-    DBSession.configure(bind=engine)
     Base.metadata.create_all(engine)
 
     print('The database "%s" has been successfully initialized' % engine.url)
@@ -126,13 +164,16 @@ def initialize_db(args):
     from alembic import command
 
     cfg = Config(args.config, ini_section="app:main")
+    if args.use_default_alembic_dir:
+        print("Will use 'script_location' from the package")
+        cfg.set_main_option('script_location', get_alembic_dir())
     command.stamp(cfg, "head")
 
 
 def upgrade_db(args):
     "Upgrade the database structure"
 
-    if not exists(args.config):
+    if not exists(args.config):  # pragma: no cover
         print('The config file "%s" does not exist!' % args.config)
         return 128
 
@@ -142,9 +183,11 @@ def upgrade_db(args):
     setup_logging(args.config)
     settings = get_appsettings(args.config)
     engine = engine_from_config(settings, 'sqlalchemy.')
-    DBSession.configure(bind=engine)
 
     cfg = Config(args.config, ini_section="app:main")
+    if args.use_default_alembic_dir:
+        print("Will use 'script_location' from the package")
+        cfg.set_main_option('script_location', get_alembic_dir())
     command.upgrade(cfg, "head")
 
     print('The database "%s" has been successfully upgraded' % engine.url)
@@ -153,18 +196,25 @@ def upgrade_db(args):
 def restore_all(args):
     "Load historic data into the database, player's portraits and club's emblems as well"
 
-    if not exists(args.config):
+    if not exists(args.config):  # pragma: no cover
         print('The config file "%s" does not exist!' % args.config)
         return 128
 
     setup_logging(args.config)
     settings = get_appsettings(args.config)
     engine = engine_from_config(settings, 'sqlalchemy.')
-    DBSession.configure(bind=engine)
 
-    sasess = DBSession()
+    register_json_decoder_encoder(json_decode, json_encode)
+
+    sasess = Session(bind=engine)
+
     pdir = settings['sol.portraits_dir']
+    if not exists(pdir):  # pragma: no cover
+        makedirs(pdir, mode=0o700)
+
     edir = settings['sol.emblems_dir']
+    if not exists(edir):  # pragma: no cover
+        makedirs(edir, mode=0o700)
 
     if args.url.startswith('file://') or exists(args.url):
         if not args.url.startswith('file://'):
@@ -177,8 +227,14 @@ def restore_all(args):
 
     print("Loading backup from %s..." % backup_url)
 
-    with transaction.manager:
+    try:
         tourneys, skipped = bio.restore(sasess, pdir, edir, url=backup_url)
+        sasess.commit()
+    except Exception:
+        sasess.rollback()
+        raise
+    finally:
+        sasess.close()
 
     print("Done, %d new/updated tourneys, %d skipped/already present." % (len(tourneys),
                                                                           skipped))
@@ -194,25 +250,29 @@ def backup_all(args):
     setup_logging(args.config)
     settings = get_appsettings(args.config)
     engine = engine_from_config(settings, 'sqlalchemy.')
-    DBSession.configure(bind=engine)
 
-    sasess = DBSession()
+    register_json_decoder_encoder(json_decode, json_encode)
+
+    sasess = Session(bind=engine)
     pdir = settings['sol.portraits_dir']
     edir = settings['sol.emblems_dir']
 
     print("Saving backup to %s..." % args.location)
 
-    with transaction.manager:
+    try:
         bio.backup(sasess, pdir, edir,
                    args.location, args.keep_only_if_changed)
+        sasess.commit()
+    except Exception:
+        sasess.rollback()
+        raise
+    finally:
+        sasess.close()
 
     print("Done.")
 
 
 def player_unique_hash(firstname, lastname, nickname):
-    from hashlib import md5
-    from time import time
-
     hash = md5()
     hash.update(firstname.encode('ascii', 'ignore'))
     hash.update(lastname.encode('ascii', 'ignore'))
@@ -222,22 +282,20 @@ def player_unique_hash(firstname, lastname, nickname):
 
 
 def _evaluator(fmap, formula):
-    from math import exp
-
     def evaluator(record, fmap=fmap, formula=formula):
         locs = {}
         for fname in fmap.values():
             if fname in fmap:
                 try:
                     locs[fname] = float(record[fname])
-                except:
+                except Exception:
                     pass
         return eval(formula, {'exp': exp}, locs)
 
     test = {fname: '1' for fname in fmap.values()}
     try:
         evaluator(test)
-    except:
+    except Exception:  # pragma: nocover
         return None
     else:
         return evaluator
@@ -272,7 +330,7 @@ def load_historical_rating(args):
             print('The formula "%s" to compute the deviation is invalid!' % args.deviation)
             return 128
     else:
-        deviation = lambda dummy, rd=int(args.deviation): rd
+        deviation = lambda dummy, rd=int(args.deviation): rd  # noqa
 
     try:
         float(args.volatility)
@@ -282,7 +340,7 @@ def load_historical_rating(args):
             print('The formula "%s" to compute the volatility is invalid!' % args.volatility)
             return 128
     else:
-        volatility = lambda dummy, rd=args.volatility: rd
+        volatility = lambda dummy, rd=args.volatility: rd  # noqa
 
     if args.rate is not None:
         rate = _evaluator(fmap, args.rate)
@@ -290,16 +348,14 @@ def load_historical_rating(args):
             print('The formula "%s" to compute the rate is invalid!' % args.rate)
             return 128
     else:
-        rate = lambda record: record[fmap['rate']]
+        rate = lambda record: record[fmap['rate']]  # noqa
 
     if not args.dry_run:
         setup_logging(args.config)
 
     settings = get_appsettings(args.config)
     engine = engine_from_config(settings, 'sqlalchemy.')
-    DBSession.configure(bind=engine)
-
-    sasess = DBSession()
+    sasess = Session(bind=engine)
 
     if not args.dry_run:
         if sasess.query(Rating).filter_by(description=args.description).first():
@@ -317,15 +373,18 @@ def load_historical_rating(args):
     new = set()
     done = set()
     rates = []
-    with transaction.manager, urlopen(args.url) as csvfile:
-        data = csvfile.read().decode(args.encoding)
-        lines = data.splitlines()
-        columns = [c.strip() for c in lines.pop(0).split(separator)]
-        for c in fmap.values():
-            if not c in columns:
-                print('Column "%s" not found!' % c)
-                return 128
 
+    with urlopen(args.url) as csvfile:
+        data = csvfile.read().decode(args.encoding)
+
+    lines = data.splitlines()
+    columns = [c.strip() for c in lines.pop(0).split(separator)]
+    for c in fmap.values():
+        if c not in columns:
+            print('Column "%s" not found!' % c)
+            return 128
+
+    try:
         for record in (dict(zip(columns, row.split(separator))) for row in lines):
             firstname = normalize(record[fmap['firstname']].strip())
             lastname = normalize(record[fmap['lastname']].strip())
@@ -378,12 +437,16 @@ def load_historical_rating(args):
                             rates=rates)
             sasess.add(rating)
 
-            print("Done, %d new rates." % len(rates))
+        sasess.commit()
+        print("Done, %d new rates." % len(rates))
+    except Exception:
+        sasess.rollback()
+        raise
+    finally:
+        sasess.close()
 
 
 def _sound(which, ogg):
-    from shutil import copyfile
-
     if not exists(ogg):
         print('Specified sound file "%s" does not exist!' % ogg)
         return 128
@@ -415,9 +478,12 @@ def prealarm_sound(args):
     return _sound('prealarm', args.sound)
 
 
-def main():
-    import sys
+def get_alembic_dir():
+    """Return the default ``script_location``"""
+    return join(dirname(dirname(realpath(sol.__file__))), 'alembic')
 
+
+def main(args=None):
     parser = ArgumentParser(description="SoL command line admin utility",
                             epilog=('You can get individual commands help'
                                     ' with "soladmin sub-command -h".'))
@@ -433,24 +499,36 @@ def main():
     subp.add_argument('-d', '--data-dir', default='%(here)s',
                       help="Directory where data (db, emblems, portraits, backups and logs)"
                       " will be stored, by default the one containing the configuration file")
+    # For test purpose, the location of the alembic scripts
+    subp.add_argument('--alembic-dir', help=SUPPRESS)
     subp.add_argument('config', help="Name of the new configuration file")
     subp.set_defaults(func=create_config)
 
     subp = subparsers.add_parser('update-config',
                                  help=update_config.__doc__)
     subp.add_argument('-a', '--admin',
-                      help="Specify the new name of the super user")
-    subp.add_argument('-p', '--password', help="Specify the new admin password")
+                      help="Specify the new name of the super user, with the value “ask”"
+                      " it will be prompted interactively")
+    subp.add_argument('-p', '--password', help="Specify the new admin password, with the value"
+                      " “ask” it will be prompted interactively")
+    # For test purpose, the location of the alembic scripts
+    subp.add_argument('--alembic-dir', help=SUPPRESS)
     subp.add_argument('config', help="Name of the existing configuration file")
     subp.set_defaults(func=update_config)
 
     subp = subparsers.add_parser('initialize-db',
                                  help=initialize_db.__doc__)
+    subp.add_argument('--use-default-alembic-dir', default=False, action='store_true',
+                      help="Override the 'script_location' parameter in the file"
+                      " with the default of the package ({})".format(get_alembic_dir()))
     subp.add_argument('config', help="Name of the configuration file")
     subp.set_defaults(func=initialize_db)
 
     subp = subparsers.add_parser('upgrade-db',
                                  help=upgrade_db.__doc__)
+    subp.add_argument('--use-default-alembic-dir', default=False, action='store_true',
+                      help="Override the 'script_location' parameter in the file"
+                      " with the default of the package ({})".format(get_alembic_dir()))
     subp.add_argument('config', help="Name of the configuration file")
     subp.set_defaults(func=upgrade_db)
 
@@ -461,10 +539,10 @@ def main():
                       " i.e. only those with at least one played match. This"
                       " flag restores all tourneys.")
     subp.add_argument('config', help="Name of the configuration file")
-    subp.add_argument('url', default="http://sol3.arstecnica.it/",
+    subp.add_argument('url', default="https://sol4.metapensiero.it/",
                       nargs="?",
                       help='URL from where historic data will be loaded'
-                      ' if different from "http://sol3.arstecnica.it/".'
+                      ' if different from "https://sol4.metapensiero.it/".'
                       ' It may also be a file:// URI or a local file path'
                       ' name.')
     subp.set_defaults(func=restore_all)
@@ -507,8 +585,8 @@ def main():
                       help="Whether player's rate will be inherited from other"
                       " ratings at the same level or better, False by default")
     subp.add_argument('--map', action='append', default=[],
-                        help="Specify a map between internal (SoL) field name and"
-                        " external one")
+                      help="Specify a map between internal (SoL) field name and"
+                      " external one")
     subp.add_argument('--encoding', default='utf-8',
                       help="Encoding of the CSV file, by default UTF-8")
     subp.add_argument('--tsv', default=False, action="store_true",
@@ -536,8 +614,8 @@ def main():
     subp.add_argument('sound', help="Name of the new OGG sound file")
     subp.set_defaults(func=prealarm_sound)
 
-    args = parser.parse_args()
-    sys.exit(args.func(args) if hasattr(args, 'func') else 0)
+    args = parser.parse_args(args)
+    sys.exit((args.func(args) or 0) if hasattr(args, 'func') else 0)
 
 
 if __name__ == '__main__':
