@@ -1,8 +1,9 @@
-//-*- coding: utf-8 -*-
-//:Progetto:  SoL -- Rated players window
-//:Creato:    dom 15 dic 2013 18:08:06 CET
-//:Autore:    Lele Gaifax <lele@metapensiero.it>
-//:Licenza:   GNU General Public License version 3 or later
+// -*- coding: utf-8 -*-
+// :Project:   SoL -- Rated players window
+// :Created:   dom 15 dic 2013 18:08:06 CET
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2013, 2014, 2016, 2018, 2021 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -23,12 +24,12 @@ Ext.define('SoL.module.RatedPlayers.Actions', {
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
+        var me = this,
+            ids = me.statics();
 
         me.callParent();
 
-        me.showChartAction = me.addAction(new Ext.Action({
+        me.showChart = me.addAction(new Ext.Action({
             itemId: ids.SHOW_CHART_ACTION,
             text: _('Chart'),
             tooltip: _('Show the rating chart of selected players.'),
@@ -36,10 +37,10 @@ Ext.define('SoL.module.RatedPlayers.Actions', {
             needsSelectedRow: true,
             disabled: true,
             handler: function() {
-                var idrating = me.module.idrating;
-                var sels = me.component.getSelectionModel().getSelection();
-                var players = [];
-                var desktop = me.module.app.getDesktop();
+                var idrating = me.module.idrating,
+                    sels = me.component.getSelectionModel().getSelection(),
+                    players = [],
+                    desktop = me.module.app.getDesktop();
 
                 for(var i=0, l=sels.length; i<l; i++) {
                     var id = sels[i].get('idplayer');
@@ -56,14 +57,14 @@ Ext.define('SoL.module.RatedPlayers.Actions', {
             }
         }));
 
-        me.showRankingAction = me.addAction(new Ext.Action({
+        me.showRanking = me.addAction(new Ext.Action({
             itemId: ids.SHOW_RANKING_ACTION,
             text: _('Print'),
             tooltip: _('Print this rating ranking.'),
-            iconCls: 'print-championship-icon',
+            iconCls: 'print-icon',
             handler: function() {
-                var idrating = me.module.idrating;
-                var url = '/pdf/ratingranking/' + idrating;
+                var idrating = me.module.idrating,
+                    url = '/pdf/ratingranking/' + idrating;
                 window.open(url, "_blank");
             }
         }));
@@ -75,7 +76,7 @@ Ext.define('SoL.module.RatedPlayers.Actions', {
         me.callParent();
 
         var tbar = me.component.child('#ttoolbar');
-        tbar.add(2, ' ', me.showChartAction, me.showRankingAction);
+        tbar.add(2, ' ', me.showChart, me.showRanking);
     }
 });
 
@@ -107,8 +108,8 @@ Ext.define('SoL.module.RatedPlayers', {
     },
 
     getConfig: function(callback) {
-        var me = this;
-        var cfg = me.config;
+        var me = this,
+            cfg = me.config;
 
         if(!cfg.metadata) {
             MP.data.MetaData.fetch(cfg.dataURL, me, function(metadata) {
@@ -141,10 +142,10 @@ Ext.define('SoL.module.RatedPlayers', {
     },
 
     createOrShowWindow: function(idrating, rating) {
-        var me = this;
-        var config = me.config;
-        var desktop = me.app.getDesktop();
-        var win = desktop.getWindow(me.id);
+        var me = this,
+            config = me.config,
+            desktop = me.app.getDesktop(),
+            win = desktop.getWindow(me.id);
 
         // If the window is already present, destroy and recreate it,
         // to reapply configuration and filters
@@ -162,7 +163,8 @@ Ext.define('SoL.module.RatedPlayers', {
                 config = Ext.apply({
                     stickyFilters: [{
                         property: 'idrating',
-                        value: idrating
+                        value: idrating,
+                        operator: '='
                     }]
                 }, config);
 
@@ -182,15 +184,18 @@ Ext.define('SoL.module.RatedPlayers', {
                         type: 'help',
                         tooltip: _('Show user manual section.'),
                         callback: function() {
-                            var whsize = desktop.getReasonableWindowSize(800, 640);
-                            var wh = Ext.create('SoL.window.Help', {
-                                width: whsize.width,
-                                height: whsize.height,
-                                // TRANSLATORS: this is the URL of the manual
-                                // page explaining players management
-                                help_url: _('/static/manual/en/playersrates.html'),
-                                title: _('Help on rated players window')
-                            });
+                            var whsize = desktop.getReasonableWindowSize(800, 640),
+                                wh = Ext.create('SoL.window.Help', {
+                                    width: whsize.width,
+                                    height: whsize.height,
+                                    // TRANSLATORS: this is the URL of the manual page
+                                    // explaining players rates management, do not change
+                                    // unless the manual is actually translated in the target
+                                    // language
+                                    help_url: _('/static/manual/en/playersrates.html'),
+                                    title: _('Help on rated players window')
+                                });
+
                             wh.show();
                         }
                     }]
