@@ -1,8 +1,9 @@
 // -*- coding: utf-8 -*-
-// :Progetto:  SoL -- Competitors panel of tourney management
-// :Creato:    gio 20 nov 2008 18:22:40 CET
-// :Autore:    Lele Gaifax <lele@metapensiero.it>
-// :Licenza:   GNU General Public License version 3 or later
+// :Project:   SoL -- Competitors panel of tourney management
+// :Created:   gio 20 nov 2008 18:22:40 CET
+// :Author:    Lele Gaifax <lele@metapensiero.it>
+// :License:   GNU General Public License version 3 or later
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -24,9 +25,9 @@ Ext.define('SoL.view.Competitors.Actions', {
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
-        var tourney = me.module.tourney;
+        var me = this,
+            ids = me.statics(),
+            tourney = me.module.tourney;
 
         me.callParent();
 
@@ -66,13 +67,13 @@ Ext.define('SoL.view.Competitors.Actions', {
     },
 
     attachActions: function() {
-        var me = this;
-        var tourney = me.module.tourney;
+        var me = this,
+            tourney = me.module.tourney;
 
         me.callParent();
 
-        var panel = me.component.up('panel');
-        var tbar = panel.child('#ttoolbar');
+        var panel = me.component.up('panel'),
+            tbar = panel.child('#ttoolbar');
 
         if(!tourney.readOnly) {
             tbar.add(0, me.addCompetitorAction, ' ',
@@ -117,7 +118,8 @@ Ext.define('SoL.view.Competitors.Actions', {
 Ext.define('SoL.view.Competitors.PlayersActions', {
     extend: 'MP.action.StoreAware',
     uses: [
-        'Ext.Action'
+        'Ext.Action',
+        'SoL.module.Tourneys'
     ],
 
     statics: {
@@ -126,8 +128,8 @@ Ext.define('SoL.view.Competitors.PlayersActions', {
     },
 
     initActions: function() {
-        var me = this;
-        var ids = me.statics();
+        var me = this,
+            ids = me.statics();
 
         me.callParent();
 
@@ -147,7 +149,7 @@ Ext.define('SoL.view.Competitors.PlayersActions', {
         me.toggleShowAllPlayersAction = me.addAction(new Ext.Action({
             itemId: ids.TOGGLE_SHOW_ALL_PLAYERS_ACTION,
             text: _('Show all players'),
-            tooltip: _('Toggle between all players or only active ones, whose partecipated to at least a tourney in the last year.'),
+            tooltip: _('Toggle between all players or only active ones, whose participated to at least a tourney organized by the current club in the last year.'),
             iconCls: 'magnify-icon',
             handler: function() {
                 me.toggleShowAllPlayers();
@@ -174,42 +176,58 @@ Ext.define('SoL.view.Competitors.PlayersActions', {
     },
 
     toggleShowAllPlayers: function(setup) {
-        var me = this;
-        var store = me.component.store;
+        var me = this,
+            component = me.component,
+            store = component.store,
+            tourney = me.competitors.ownerCt.module.tourney,
+            win = component.up();
+
+        if(!component.orig_dataURL)
+            component.orig_dataURL = component.dataURL;
 
         if(setup || !me.show_active_players) {
             me.toggleShowAllPlayersAction.setText(_('Show all players'));
             me.show_active_players = true;
-            var today = new Date();
-            var oneyearago = new Date(today.getFullYear()-1,
+
+            var today = new Date(),
+                oneyearago = new Date(today.getFullYear()-1,
                                       today.getMonth(),
                                       today.getDate());
+
+            store.proxy.url = Ext.String.urlAppend(component.orig_dataURL,
+                                                   'played4club=' + tourney.IDClub);
+
             store.stickyFilter({
                 id: 'lastplayed',
                 property: 'LastPlayed',
-                value: oneyearago,
+                value: SoL.module.Tourneys.dateToISO(oneyearago),
                 operator: ">="
             });
+
+            win.setTitle(Ext.String.format(_("Players of last year's “{0}” tourneys"),
+                                           tourney.Club));
         } else {
             me.toggleShowAllPlayersAction.setText(_('Show only active players'));
             me.show_active_players = false;
+            store.proxy.url = component.orig_dataURL;
             store.stickyFilters.removeAtKey('lastplayed');
             store.filters.removeAtKey('lastplayed');
             store.load();
+            win.setTitle(_('All players'));
         }
     },
 
     insertSelectedPlayers: function() {
-        var me = this;
-        var grid = me.component;
-        var sels = grid.getSelectionModel().getSelection();
-        var competitor;
-        var nplayers = 0;
-        var tourney = me.competitors.ownerCt.module.tourney;
-        var idtourney = tourney.idtourney;
-        var playersperteam = tourney.PlayersPerTeam;
-        var deststore = me.competitors.store;
-        var ids = me.player_ids;
+        var me = this,
+            grid = me.component,
+            sels = grid.getSelectionModel().getSelection(),
+            competitor,
+            nplayers = 0,
+            tourney = me.competitors.ownerCt.module.tourney,
+            idtourney = tourney.idtourney,
+            playersperteam = tourney.PlayersPerTeam,
+            deststore = me.competitors.store,
+            ids = me.player_ids;
 
         Ext.each(sels, function(record) {
             var idplayer = record.get('idplayer');
@@ -335,18 +353,17 @@ Ext.define('SoL.view.Competitors', {
     statics: {
         getConfig: function(callback, errorcb, config) {
             //jsl:unused errorcb
-            var me = this;
-            var dataURL = '/tourney/competitors';
-
-            var dataview = {
-                xtype: 'competitors-dataview',
-                id: 'competitors-view',
-                plugins: [
-                    Ext.create('SoL.view.Competitors.Actions', {
-                        module: me
-                    })
-                ]
-            };
+            var me = this,
+                dataURL = '/tourney/competitors',
+                dataview = {
+                    xtype: 'competitors-dataview',
+                    id: 'competitors-view',
+                    plugins: [
+                        Ext.create('SoL.view.Competitors.Actions', {
+                            module: me
+                        })
+                    ]
+                };
 
             if(config.tourney.PlayersPerTeam > 1 &&
                !config.tourney.prized &&
@@ -363,13 +380,14 @@ Ext.define('SoL.view.Competitors', {
                             getDragData: function(e) {
                                 var player = e.getTarget('span.player', 10, true);
                                 if (player) {
-                                    var container = player.parent();
-                                    var cid = container.dom.attributes.cid.nodeValue;
-                                    var idcompetitor = parseInt(cid, 10);
-                                    var competitor;
-                                    var xy = player.getXY();
-                                    var sourceEl = player.dom;
-                                    var d = sourceEl.cloneNode(true);
+                                    var container = player.parent(),
+                                        cid = container.dom.attributes.cid.nodeValue,
+                                        idcompetitor = parseInt(cid, 10),
+                                        competitor,
+                                        xy = player.getXY(),
+                                        sourceEl = player.dom,
+                                        d = sourceEl.cloneNode(true);
+
                                     d.id = Ext.id();
                                     if(isNaN(idcompetitor)) {
                                         // the node contains a phantom record
@@ -426,12 +444,12 @@ Ext.define('SoL.view.Competitors', {
             };
 
             MP.data.MetaData.fetch(dataURL, me, function(metadata) {
-                var overrides = {
-                    rate: {
-                        sendback: false
-                    }
-                };
-                var model;
+                var model,
+                    overrides = {
+                        rate: {
+                            sendback: false
+                        }
+                    };
 
                 model = Ext.define('MP.data.ImplicitModel-' + Ext.id(), {
                     extend: 'Ext.data.Model',
@@ -472,8 +490,8 @@ Ext.define('SoL.view.Competitors', {
     },
 
     initEvents: function() {
-        var me = this;
-        var tourney = me.ownerCt.module.tourney;
+        var me = this,
+            tourney = me.ownerCt.module.tourney;
 
         me.on({
             itemdblclick: function(view, competitor) {
@@ -483,10 +501,8 @@ Ext.define('SoL.view.Competitors', {
                     return;
                 }
 
-                var players = [];
-                var he = competitor.get('player1Sex') == 'M'
-                    ? _('he')
-                    : _('she');
+                var players = [],
+                    he = competitor.get('player1Sex') == 'M' ? _('he') : _('she');
 
                 for(var i=1; i <= 4; i++) {
                     var fn = competitor.get('player' + i + 'FullName');
@@ -520,10 +536,10 @@ Ext.define('SoL.view.Competitors', {
     },
 
     prepareData: function(data, index, record) {
-        var me = this;
-        var tourney = me.ownerCt.module.tourney;
-        var ppt = tourney.PlayersPerTeam;
-        var dirty;
+        var me = this,
+            tourney = me.ownerCt.module.tourney,
+            ppt = tourney.PlayersPerTeam,
+            dirty;
 
         me.callParent(data, index, record);
 
@@ -559,10 +575,10 @@ Ext.define('SoL.view.Competitors', {
     },
 
     showOtherPlayers: function() {
-        var me = this;
-        var otheractions;
-        var module = me.ownerCt.module.app.getModule('players-win');
-        var idp, ids = [];
+        var me = this,
+            otheractions,
+            module = me.ownerCt.module.app.getModule('players-win'),
+            idp, ids = [];
 
         me.store.each(function(comp) {
             ids.push(comp.get('idplayer1'));
@@ -581,7 +597,7 @@ Ext.define('SoL.view.Competitors', {
             competitors: me,
             player_ids: ids
         });
-        module.createOrShowWindow(ids, otheractions);
+        module.createOrShowWindow('competitors', ids, otheractions);
     },
 
     commitChanges: function(callback, scope) {
@@ -620,8 +636,8 @@ Ext.define('SoL.view.Competitors.PlayerDrop', {
     },
 
     isAnyPlayerAlreadyPresent: function(sels) {
-        var me = this;
-        var present = false;
+        var me = this,
+            present = false;
 
         function is_present(comp, id) {
             if(id == comp.get("idplayer1") ||
@@ -643,8 +659,8 @@ Ext.define('SoL.view.Competitors.PlayerDrop', {
     },
 
     notifyEnter: function(source, e, data) {
-        var me = this;
-        var r;
+        var me = this,
+            r;
 
         //<debug>
         console.debug('Checking droppability...');
@@ -660,9 +676,9 @@ Ext.define('SoL.view.Competitors.PlayerDrop', {
 
     notifyOver: function(source, e, data) {
         //jsl:unused source
-        var me = this;
-        var target = e.getTarget('.competitor');
-        var idplayer1, index, comp, nplayers;
+        var me = this,
+            target = e.getTarget('.competitor'),
+            idplayer1, index, comp, nplayers;
 
         if(me.tourney.readOnly) {
             //<debug>
@@ -759,11 +775,10 @@ Ext.define('SoL.view.Competitors.PlayerDrop', {
 
     notifyDrop : function(source, e, data) {
         //jsl:unused source
-        var me = this;
-        var target = e.getTarget('.competitor');
-        var idplayer, idplayer1, index, comp, nplayers, spos, others, sc, i;
-        var full_name = SoL.module.Players.full_name;
-        var idp;
+        var me = this,
+            target = e.getTarget('.competitor'),
+            full_name = SoL.module.Players.full_name,
+            idplayer, idplayer1, index, comp, nplayers, spos, others, sc, i, idp;
 
         if(me.tourney.readOnly) {
             //<debug>
