@@ -3,7 +3,7 @@
 // :Created:   gio 20 nov 2008 18:23:54 CET
 // :Author:    Lele Gaifax <lele@metapensiero.it>
 // :License:   GNU General Public License version 3 or later
-// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020, 2021 Lele Gaifax
+// :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2020, 2021, 2022 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -177,7 +177,6 @@ Ext.define('SoL.view.Matches.Actions', {
             tooltip: _('Show or edit per-board results of the selected match.'),
             iconCls: 'edit-record-icon',
             disabled: true,
-            hidden: tourney.readOnly,
             needsOneSelectedRow: true,
             handler: function() {
                 var record = me.component.getSelectionModel().getSelection()[0];
@@ -326,10 +325,14 @@ Ext.define('SoL.view.Matches.Actions', {
         if(act.itemId == statics.EDIT_MATCH_ACTION) {
             var tourney = me.module.tourney,
                 grid = me.component;
-            disable = (tourney.prized
-                       || tourney.readOnly
-                       || !tourney.currentturn
+            disable = (!tourney.currentturn
                        || grid.getSelectionModel().getSelection().length != 1);
+            // Hide the action, when the tourney is over and no board details
+            if(!disable
+               && (tourney.prized || tourney.readOnly)
+               && grid.getSelectionModel().getSelection()[0].get('coins1_1') === null) {
+                disable = true;
+            }
         } else if(act.itemId == statics.REFRESH_ACTION) {
             disable = me.module.matches_grid.store.shouldDisableAction(act);
         }
@@ -622,7 +625,7 @@ Ext.define('SoL.view.Matches', {
         this.callParent();
     },
 
-    newTurn: function() {
+    _newTurn: function(url) {
         var me = this,
             ordinalp = me.statics().ordinalp,
             tourney = me.module.tourney,
@@ -639,7 +642,7 @@ Ext.define('SoL.view.Matches', {
         }
 
         Ext.Ajax.request({
-            url: '/tourney/newTurn',
+            url: url,
             params: { idtourney: tourney.idtourney },
             success: function (r) {
                 var res = Ext.decode(r.responseText);
@@ -652,6 +655,7 @@ Ext.define('SoL.view.Matches', {
                         var cturn = res.currentturn;
                         tourney.currentturn = cturn;
                         tourney.rankedturn = res.rankedturn;
+                        tourney.finalturns = res.finalturns;
                         tourney.prized = res.prized;
                         tourney.GeneratedTurns = res.generated_turns;
 
@@ -679,59 +683,12 @@ Ext.define('SoL.view.Matches', {
         });
     },
 
+    newTurn: function() {
+        this._newTurn('/tourney/newTurn');
+    },
+
     finalTurn: function() {
-        var me = this,
-            ordinalp = me.statics().ordinalp,
-            tourney = me.module.tourney,
-            lbar = me.child('toolbar[dock="left"]');
-
-        me.allowEditPreviousTurns = false;
-
-        if(me.focusedCompetitor) {
-            lbar.show();
-            me.getColumnByName('turn').hide();
-            me.getColumnByName('board').show();
-            me.filterOnTurn(tourney.currentturn);
-            me.focusedCompetitor = null;
-        }
-
-        Ext.Ajax.request({
-            url: '/tourney/finalTurn',
-            params: { idtourney: tourney.idtourney },
-            success: function (r) {
-                var res = Ext.decode(r.responseText);
-                if(!res) {
-                    Ext.MessageBox.alert(
-                        _("Comunication error"),
-                        _('Cannot decode JSON object'));
-                } else {
-                    if(res.success) {
-                        var cturn = res.currentturn;
-
-                        tourney.currentturn = cturn;
-                        tourney.rankedturn = res.rankedturn;
-                        tourney.finalturns = res.finalturns;
-                        tourney.prized = res.prized;
-
-                        lbar.insert(cturn-1, Ext.create('Ext.button.Button', {
-                            itemId: 'turn-' + cturn,
-                            text: cturn,
-                            tooltip: Ext.String.format(
-                                _('Show the matches {0} round.'),
-                                ordinalp(cturn)),
-                            turn: cturn,
-                            handler: function(btn) {
-                                me.filterOnTurn(btn.turn);
-                            }
-                        }));
-                        me.filterOnTurn(cturn);
-                        me.updateActions();
-                    } else {
-                        Ext.MessageBox.alert(_('Error'), res.message);
-                    }
-                }
-            }
-        });
+        this._newTurn('/tourney/finalTurn');
     },
 
     deleteTurn: function(turn) {
@@ -1178,8 +1135,8 @@ Ext.define('SoL.view.Matches', {
             qcolumn = columns[1];
 
         for(var i = 1; i < 10; i++) {
-            var coins1 = c1column.getComponent('coins1_' + i).getValue(),
-                coins2 = c2column.getComponent('coins2_' + i).getValue(),
+            var coins1 = c1column.getComponent('coins1_' + i).getValue() || 0,
+                coins2 = c2column.getComponent('coins2_' + i).getValue() || 0,
                 queen = qcolumn && qcolumn.getComponent('queen_' + i).getValue();
 
             if(coins1 && coins2) {
@@ -1188,23 +1145,23 @@ Ext.define('SoL.view.Matches', {
                 break;
             }
 
-            if(coins1 !== null || coins2 !== null) {
-                if(coins1 === null)
-                    coins1 = 0;
-                else if(coins2 === null)
-                    coins2 = 0;
+            if(coins1)
+                total1 += coins1;
 
-                if(coins1 > coins2) {
-                    if(queen == '1')
-                        if(total1 < 22)
-                            total1 += 3;
-                    total1 += coins1;
-                } else if(coins1 < coins2) {
-                    if(queen == '2')
-                        if(total2 < 22)
-                            total2 += 3;
-                    total2 += coins2;
-                }
+            if(coins2)
+                total2 += coins2;
+
+            // Did one of the player commit suicide?
+            if(coins1 == 0 && coins2 == 0) {
+                if(queen == '1')
+                    total1 += total1 < 22 ? 3 : 1;
+                else if(queen == '2')
+                    total2 += total2 < 22 ? 3 : 1;
+            } else {
+                if(queen == '1' && coins1 > coins2 && total1 < 22)
+                    total1 += 3;
+                else if(queen == '2' && coins1 < coins2 && total2 < 22)
+                    total2 += 3;
             }
         }
 
