@@ -3,7 +3,7 @@
 // :Created:   lun 16 lug 2018 07:59:55 CEST
 // :Author:    Lele Gaifax <lele@metapensiero.it>
 // :License:   GNU General Public License version 3 or later
-// :Copyright: © 2018, 2020 Lele Gaifax
+// :Copyright: © 2018, 2020, 2022 Lele Gaifax
 //
 
 /*jsl:declare Ext*/
@@ -45,7 +45,7 @@ Ext.define('SoL.window.Signin', {
     },
 
     buttons: ['->', {
-        text: _("Sign in"),
+        text: _("Register"),
         type: "submit",
         formBind: true,
         handler: function(button) {
