@@ -3,7 +3,7 @@
 # :Created:   lun 15 apr 2013 16:48:23 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2019, 2020 Lele Gaifax
+# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2022 Lele Gaifax
 #
 
 from datetime import datetime
@@ -493,11 +493,33 @@ REGISTRATION_MESSAGE = _("""\
 Hello again,
 
 this to confirm that your registration has been successfully completed: you
-can now access your own account at {login_url} and start using SoL.
+can now access your own account at
+
+  {login_url}
+
+and start using SoL.
 
 Use this email address as the “username” and provide the same password you
 specified in the request.
 
+Please consult the online documentation at
+
+  {manual_url}
+
+in particular the section about players management at
+
+  {players_mgmt_url}
+
+If you need help, or would like to register players not already present
+in the database, either send an email to the administrator at the following
+address
+
+  {admin_email}
+
+or subscribe to the very low traffic "SoL users" mailing list and ask there
+
+  https://groups.google.com/g/sol-users
+
 Best regards and happy carromming!
 """)
 
@@ -539,7 +561,18 @@ def confirm_new_user(request):
     t = translator(request)
 
     login_url = request.route_url('app')
-    body = t(REGISTRATION_MESSAGE).format(login_url=login_url)
+    # TRANSLATORS: this is the route to the user's manual, leave it UNCHANGED
+    # if the manual is NOT translated in your language!
+    manual_url = request.static_url(t(_('sol:static/manual/en/index.html')))
+    # TRANSLATORS: this is the route to the players manager section in the
+    # user's manual, leave it UNCHANGED if the manual is NOT translated in
+    # your language!
+    players_mgmt_url = request.static_url(t(_('sol:static/manual/en/users.html#players-manager')))
+    admin_email = request.registry.settings.get('sol.admin.email', 'lele@metapensiero.it')
+    body = t(REGISTRATION_MESSAGE).format(login_url=login_url,
+                                          manual_url=manual_url,
+                                          players_mgmt_url=players_mgmt_url,
+                                          admin_email=admin_email)
     message = Message(subject=t(_('Your new SoL account has been activated')),
                       recipients=[user.email],
                       body=body)
