@@ -3,13 +3,13 @@
 # :Created:   mer 15 ott 2008 08:25:21 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021 Lele Gaifax
+# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2015, 2016, 2018, 2019, 2020, 2021, 2022 Lele Gaifax
 #
 
 import logging
 
+from metapensiero.sqlalchemy.proxy.filters import Operator, extract_filters
 from pyramid.view import view_config
-
 from sqlalchemy import and_, bindparam, distinct, exists, func, or_, select
 from sqlalchemy.orm import load_only
 
@@ -486,7 +486,25 @@ def players():
             ),)
             results = yield args, conditions
         else:
-            results = yield args
+            # If there is a single filter on either the first name or the last name transform
+            # it in an expression that checks for the presence of the search term in either one
+            # of those fields
+            if 'filter' in args:
+                # Save it, as extract_filter() removes them from the dictionary
+                old_args = dict(args)
+                filters = extract_filters(args)
+                if ((len(filters) == 1
+                     and filters[0].property in {'firstname', 'lastname'}
+                     and filters[0].operator == Operator.CONTAINS)):
+                    op = filters[0].operator
+                    value = filters[0].value
+                    conditions = or_(op.filter(_players_t.c.firstname, value),
+                                     op.filter(_players_t.c.lastname, value))
+                    results = yield args, (conditions,)
+                else:
+                    results = yield old_args
+            else:
+                results = yield args
         for r in results['root']:
             code = r['language']
             if code:
@@ -597,9 +615,13 @@ def owners(request, results):
         lastlogin=dict(hidden=True, width=140),
         ownersadmin=dict(hidden=True, nullable=True),
         playersmanager=dict(hidden=True, nullable=True)))
-def users(request, results):
+def users():
+    request, args = (yield)
+
     t = translator(request)
-    if 'metadata' in results:
+
+    if 'metadata' in args:
+        results = yield args
         results['metadata']['fields'].append({
             'label': t(_('Language')),
             'hint': t(_('The preferred language of the user, used to send email messages'
@@ -614,12 +636,33 @@ def users(request, results):
                            displayField='name'),
         })
     else:
+        # If there is a single filter on either the first name, the last name or the email
+        # transform it in an expression that checks for the presence of the search term in
+        # either one of those fields
+        if 'filter' in args:
+            # Save it, as extract_filter() removes them from the dictionary
+            old_args = dict(args)
+            filters = extract_filters(args)
+            if ((len(filters) == 1
+                 and filters[0].property in {'email', 'firstname', 'lastname'}
+                 and filters[0].operator == Operator.CONTAINS)):
+                op = filters[0].operator
+                value = filters[0].value
+                conditions = or_(op.filter(_users_t.c.email, value),
+                                 op.filter(_users_t.c.firstname, value),
+                                 op.filter(_users_t.c.lastname, value))
+                results = yield args, (conditions,)
+            else:
+                results = yield old_args
+        else:
+            results = yield args
+
         for r in results['root']:
             r['password'] = '*' * 6
             code = r['language']
             if code:
                 r['Language'] = language_name(code)
-    return results
+    yield results
 
 
 _pchampionships_t = _championships_t.alias()
