@@ -3,7 +3,7 @@
 # :Created:   ven 12 dic 2008 09:18:37 CET
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2016, 2018, 2020, 2021 Lele Gaifax
+# :Copyright: © 2008–2010, 2013, 2014, 2016, 2018, 2020–2023 Lele Gaifax
 #
 
 from datetime import date
@@ -11,6 +11,7 @@ from functools import cmp_to_key, partial, wraps
 from itertools import groupby
 from operator import itemgetter
 import logging
+import time
 
 from babel.numbers import format_decimal
 from itsdangerous import BadData, Signer
@@ -20,6 +21,7 @@ from pyramid.httpexceptions import (
     HTTPFound,
     HTTPMovedPermanently,
     HTTPNotFound,
+    HTTPSeeOther,
     )
 from pyramid.view import view_config
 from pyramid.settings import asbool
@@ -428,8 +430,44 @@ def tourney(request, session, entities):
                 "Couldn't create page: argument “turn” is not an integer: %r", turn)
             e = t(_('Invalid turn: $turn'), mapping=dict(turn=repr(turn)))
             raise HTTPBadRequest(str(e))
+        else:
+            if turn > tourney.currentturn:
+                get_request_logger(request, logger).warning(
+                    "Couldn't create page: argument “turn” is invalid: %r", turn)
+                e = t(_('Invalid turn: $turn'), mapping=dict(turn=repr(turn)))
+                raise HTTPBadRequest(str(e))
 
-    data = _build_template_data(request, session, tourney, turn=turn,
+        board = request.params.get('board')
+        if board is not None:
+            try:
+                board = int(board)
+            except ValueError:
+                get_request_logger(request, logger).warning(
+                    "Couldn't create page: argument “board” is not an integer: %r", board)
+                e = t(_('Invalid board number: $board'), mapping=dict(board=repr(board)))
+                raise HTTPBadRequest(str(e))
+            else:
+                sess = request.dbsession
+                match = sess.query(Match).filter_by(idtourney=tourney.idtourney, turn=turn,
+                                                    board=board).one_or_none()
+                if match is None:
+                    get_request_logger(request, logger).warning(
+                        "Couldn't create page: argument “board” is invalid: %r", board)
+                    e = t(_('Invalid board number: $board'), mapping=dict(board=repr(board)))
+                    raise HTTPBadRequest(str(e))
+
+                match.compute_partial_scores()
+        else:
+            for match in tourney.matches:
+                if match.turn == turn:
+                    match.compute_partial_scores()
+            match = None
+
+
+    else:
+        match = None
+
+    data = _build_template_data(request, session, tourney, turn=turn, match=match,
                                 player=request.params.get('player'))
 
     if tourney.championship.prizes != 'centesimal':
@@ -729,6 +767,13 @@ def match_form(request):
         return HTTPFound(location=request.route_path('lit_tourney', guid=m.tourney.guid,
                                                      _query={'turn': m.tourney.currentturn}))
 
+    if m.tourney.countdownstarted is None:
+        elapsed = None
+    else:
+        elapsed = int(time.time() * 1000) - m.tourney.countdownstarted
+        if elapsed > m.tourney.duration * 60 * 1000:  # pragma: no cover
+            elapsed = None
+
     return {
         '_': gettext,
         'escape': escape,
@@ -737,6 +782,7 @@ def match_form(request):
         'tourney': m.tourney,
         'match': m,
         'currentturn': m.tourney.currentturn,
+        'elapsed': elapsed,
         'request': request,
         'session': request.dbsession,
         'today': date.today(),
@@ -784,8 +830,9 @@ def store_match_boards(request):
     getter = request.params.get
 
     if m.score1 or m.score2 or int(getter('turn', 0)) != m.tourney.currentturn:
-        return HTTPFound(location=request.route_path('lit_tourney', guid=m.tourney.guid,
-                                                     _query={'turn': m.tourney.currentturn}))
+        return HTTPSeeOther(
+            location=request.route_path('lit_tourney', guid=m.tourney.guid,
+                                        _query={'turn': m.tourney.currentturn}))
 
     breaker = getter('breaker')
     if breaker is not None:
@@ -811,7 +858,7 @@ def store_match_boards(request):
         coins1 = 0 if coins1 is None or not coins1.isdigit() else int(coins1)
         coins2 = 0 if coins2 is None or not coins2.isdigit() else int(coins2)
 
-        if coins1 > 0 and coins2 > 0:  # pragma: no cover
+        if coins1 > 0 and coins2 > 0:
             doom()
             return {'success': False,
                     'message': translate(_('Cannot accept both coins in board $board!',
@@ -823,24 +870,22 @@ def store_match_boards(request):
                 return {'success': False,
                         'message': translate(_('Bad value for Queen in board $board!',
                                                mapping=dict(board=boardno)))}
-            board.queen = queen
 
-        if coins1 > 0:
-            if not 0 <= coins1 < 10:
-                doom()
-                return {'success': False,
-                        'message': translate(_('Out of range value for coins 1 in board'
-                                               ' $board!', mapping=dict(board=boardno)))}
-            board.coins1 = coins1
-            board.coins2 = 0
-        elif coins2 > 0:
-            if not 0 <= coins2 < 10:  # pragma: no cover
-                doom()
-                return {'success': False,
-                        'message': translate(_('Out of range value for coins 2 in board'
-                                               ' $board!', mapping=dict(board=boardno)))}
-            board.coins2 = coins2
-            board.coins1 = 0
+        if not 0 <= coins1 < 10:
+            doom()
+            return {'success': False,
+                    'message': translate(_('Out of range value for coins 1 in board'
+                                           ' $board!', mapping=dict(board=boardno)))}
+
+        if not 0 <= coins2 < 10:
+            doom()
+            return {'success': False,
+                    'message': translate(_('Out of range value for coins 2 in board'
+                                           ' $board!', mapping=dict(board=boardno)))}
+
+        board.coins1 = coins1
+        board.coins2 = coins2
+        board.queen = queen
 
     if 'endgame' in request.params:
         m.score1 = int(getter('score1'))
@@ -856,8 +901,19 @@ def store_match_boards(request):
                     'message': translate(_('Out of range value for score 2!'))}
 
         changes_logger.info('%r has been self-compiled', m)
-        return HTTPFound(location=request.route_path('lit_tourney', guid=m.tourney.guid,
-                                                     _query={'turn': m.tourney.currentturn}))
+        return HTTPSeeOther(
+            location=request.route_path('lit_tourney', guid=m.tourney.guid,
+                                        _query={'turn': m.tourney.currentturn,
+                                                'board': m.board}))
     else:
-        changes_logger.info('%r has been self-compiled', m)
-        return {'success': True, 'message': 'Ok'}
+        changes_logger.info('%r is being self-compiled', m)
+        if m.tourney.countdownstarted is None:
+            elapsed = None
+        else:
+            elapsed = int(time.time() * 1000) - m.tourney.countdownstarted
+            if elapsed > m.tourney.duration * 60 * 1000:  # pragma: no cover
+                elapsed = None
+        return {
+            'success': True, 'message': 'Ok',
+            'elapsed': elapsed
+        }
