@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
-# :Progetto:  SoL
-# :Creato:    lun 16 dic 2013 20:20:40 CET
-# :Autore:    Lele Gaifax <lele@metapensiero.it>
-# :Licenza:   GNU General Public License version 3 or later
+# :Project:   SoL
+# :Created:   lun 16 dic 2013 20:20:40 CET
+# :Author:    Lele Gaifax <lele@metapensiero.it>
+# :License:   GNU General Public License version 3 or later
+# :Copyright: © 2013, 2014, 2015, 2016, 2018, 2020 Lele Gaifax
 #
 
 from pyramid.view import view_config
@@ -13,7 +14,7 @@ from sqlalchemy import select
 from sqlalchemy.orm.exc import NoResultFound
 
 from ..i18n import translatable_string as _, gettext, ngettext, translator
-from ..models import DBSession, Player, Rate, Rating
+from ..models import Player, Rate, Rating
 
 
 @view_config(route_name="svg_ratingchart")
@@ -24,7 +25,7 @@ def ratingChart(request):
 
     t = translator(request)
 
-    sas = DBSession()
+    sas = request.dbsession
 
     try:
         idrating = int(request.matchdict['id'])
@@ -41,16 +42,16 @@ def ratingChart(request):
     if 'idplayer' in request.params:
         try:
             playerids = list(map(int, request.params.getall('idplayer')))
-        except (TypeError, ValueError):
+        except (TypeError, ValueError):  # pragma: no cover
             raise HTTPBadRequest("Bad player ids")
     else:
         playerguids = list(request.params.getall('player'))
-        if not playerguids:
+        if not playerguids:  # pragma: no cover
             raise HTTPBadRequest("Missing players guids")
         pt = Player.__table__
         playerids = [p[0] for p in sas.execute(select([pt.c.idplayer])
                                                .where(pt.c.guid.in_(playerguids)))]
-        if not playerids:
+        if not playerids:  # pragma: no cover
             raise HTTPBadRequest("Wrong players guids")
 
     width = int(request.params.get('width', 800))
@@ -134,7 +135,8 @@ def playersDistribution(request):
     from gettext import translation
     from collections import Counter
     from pycountry import LOCALES_DIR, countries
-    import pygal, pygal_maps_world
+    import pygal
+    import pygal_maps_world
 
     width = int(request.params.get('width', 800))
     height = int(request.params.get('height', 600))
@@ -143,13 +145,13 @@ def playersDistribution(request):
     try:
         ct = translation('iso3166', LOCALES_DIR, languages=[lname]).gettext
     except IOError:
-        ct = lambda x: x
+        ct = lambda x: x  # noqa
 
     pygal_countries = pygal_maps_world.i18n.COUNTRIES
     for alpha2 in pygal_countries:
         try:
-            c = countries.get(alpha2=alpha2.upper())
-        except KeyError:
+            c = countries.get(alpha_2=alpha2.upper())
+        except KeyError:  # pragma: no cover
             pass
         else:
             pygal_countries[alpha2] = ct(c.name)
@@ -159,7 +161,7 @@ def playersDistribution(request):
     chart = pygal.maps.world.World(show_legend=False, width=width, height=height)
     chart.title = t(_('Players distribution around the globe'))
 
-    sas = DBSession()
+    sas = request.dbsession
 
     pt = Player.__table__
     players = sas.execute(select([pt.c.sex, pt.c.nationality])
@@ -169,13 +171,11 @@ def playersDistribution(request):
     women = Counter()
 
     for sex, nationality in players:
-        try:
-            country = countries.get(alpha3=nationality)
-        except KeyError:
-            pass
-        else:
-            counter = women if sex == 'F' else men
-            counter[country.alpha2.lower()] += 1
+        country = countries.get(alpha_3=nationality)
+        if country is None:
+            continue
+        counter = women if sex == 'F' else men
+        counter[country.alpha_2.lower()] += 1
 
     all_keys = set(men.keys()) | set(women.keys())
     serie = {}
@@ -200,7 +200,7 @@ def opponentChart(request):
 
     t = translator(request)
 
-    sas = DBSession()
+    sas = request.dbsession
 
     pguid = request.matchdict['guid']
     oguid = request.matchdict['opponent']
@@ -239,7 +239,7 @@ def opponentChart(request):
     pdata = []
     owins = 0
     odata = []
-    for match in matches:
+    for match in matches:  # pragma: no cover
         if match.score1 != match.score2:
             if match.competitor1.player1 is player:
                 if match.score1 > match.score2:
