@@ -3,7 +3,7 @@
 # :Created:   gio 23 ott 2008 11:13:02 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2008, 2009, 2010, 2013, 2014, 2016, 2017, 2018, 2020, 2021 Lele Gaifax
+# :Copyright: © 2008–2010, 2013, 2014, 2016–2018, 2020–2023 Lele Gaifax
 #
 
 import logging
@@ -417,18 +417,17 @@ def deleteFromTurn(request):
                 prized=tourney.prized)
 
 
-@view_config(route_name="new_turn", renderer="json")
-@unauthorized_for_guest
-def newTurn(request):
-    "Create next turn, or all possible turns when coupling is “all against all”."
-
+def _new_turn(request, final):
     try:
         sess = request.dbsession
         params = request.params
         idtourney = int(params['idtourney'])
 
         tourney = sess.query(Tourney).get(idtourney)
-        tourney.makeNextTurn()
+        if final:
+            tourney.makeFinalTurn()
+        else:
+            tourney.makeNextTurn()
 
         sess.flush()
 
@@ -454,6 +453,23 @@ def newTurn(request):
                                  else max(m.turn for m in tourney.matches)))
 
 
+
+@view_config(route_name="new_turn", renderer="json")
+@unauthorized_for_guest
+def newTurn(request):
+    "Create next turn, or all possible turns when coupling is “all against all”."
+
+    return _new_turn(request, False)
+
+
+@view_config(route_name="final_turn", renderer="json")
+@unauthorized_for_guest
+def finalTurn(request):
+    "Create final turn."
+
+    return _new_turn(request, True)
+
+
 @view_config(route_name="send_training_urls", renderer="json")
 @unauthorized_for_guest
 def sendTrainingURLs(request):
@@ -470,41 +486,6 @@ def sendTrainingURLs(request):
                     message=request.localizer.translate(_("Bad request!")))
 
 
-@view_config(route_name="final_turn", renderer="json")
-@unauthorized_for_guest
-def finalTurn(request):
-    "Create final turn."
-
-    try:
-        sess = request.dbsession
-        params = request.params
-        idtourney = int(params['idtourney'])
-
-        tourney = sess.query(Tourney).get(idtourney)
-        tourney.makeFinalTurn()
-
-        sess.flush()
-
-        success = True
-        message = 'Ok'
-    except OperationAborted as e:  # pragma: no cover
-        errormsg = e.args[0]
-        get_request_logger(request, logger).error("Couldn't create final turn: %s", errormsg)
-        message = request.localizer.translate(errormsg)
-        success = False
-    except Exception as e:  # pragma: no cover
-        message = str(e)
-        get_request_logger(request, logger).critical("Couldn't create final turn: %s",
-                                                     message, exc_info=True)
-        success = False
-
-    return dict(success=success, message=message,
-                currentturn=tourney.currentturn,
-                rankedturn=tourney.rankedturn,
-                finalturns=tourney.finalturns,
-                prized=tourney.prized)
-
-
 @view_config(route_name="update_ranking", renderer="json")
 @unauthorized_for_guest
 def updateRanking(request):
@@ -693,7 +674,7 @@ def countdownStarted(request):
     try:
         idtourney = int(params['idtourney'])
         if 'start' in params:
-            start = int(params['start'])
+            start = int(time.time() * 1000)
         else:
             start = None
     except Exception as e:  # pragma: no cover
@@ -743,17 +724,19 @@ def countdown(request):
         raise HTTPNotFound("Invalid tourney ID")
 
     curruser = request.session.get('user_id', '*nouser*')
-    now = int(time.time() * 1000)
-    starttime = tourney.countdownstarted
-    if starttime and (starttime + tourney.duration * 60 * 1000) < now:  # pragma: no cover
-        starttime = None
+    if tourney.countdownstarted is None:
+        elapsed = None
+    else:
+        elapsed = int(time.time() * 1000) - tourney.countdownstarted
+        if elapsed > tourney.duration * 60 * 1000:  # pragma: no cover
+            elapsed = None
     return dict(
         _=t,
         ngettext=ngettext,
         duration=tourney.duration,
         prealarm=tourney.prealarm,
         currentturn=ordinal(tourney.currentturn),
-        starttime=starttime,
+        elapsed=elapsed,
         notifystart=request.path_qs,
         isowner=curruser == tourney.idowner)
 
