@@ -3,7 +3,7 @@
 # :Created:   ven 06 lug 2018 16:22:49 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2018, 2019, 2020, 2021 Lele Gaifax
+# :Copyright: © 2018, 2019, 2020, 2021, 2022 Lele Gaifax
 #
 
 from datetime import date, datetime
@@ -146,15 +146,15 @@ def test_dazed_iterator():
     h = FakeCompetitor('H', 8)   # 7
 
     ranking = [a, b, c, d, e, f, g, h]
-    done = set([(a, f), (f, a),
+    done = set([(a, g), (g, a),
                 (b, e), (e, b),
                 (c, d), (d, c),
-                (g, h), (h, g)])
+                (f, h), (h, f)])
 
     t = Tourney()
     order = Tourney.DazedVisitor(t, a, ranking, done)
     order = list(order)
-    expected = [c, d, e, b, g, h]
+    expected = [d, e, f, b, c, h]
     assert order == expected
 
 
@@ -271,7 +271,7 @@ def test_serial_iterator_delay_compatriots():
 
     order = Tourney.DazedVisitor(t, a, ranking, done)
     order = list(order)
-    expected = [d, c, e, b, g, h]
+    expected = [d, e, b, c, g, h]
     assert order == expected
 
 
@@ -301,7 +301,7 @@ def test_serial_iterator_delay_compatriots_odd():
 
     order = Tourney.DazedVisitor(t, a, ranking, done)
     order = list(order)
-    expected = [d, c, e, b, g, h, i, None]
+    expected = [d, e, b, c, g, h, i, None]
     assert order == expected
 
 
