@@ -3,7 +3,7 @@
 # :Created:   lun 09 lug 2018 11:11:25 CEST
 # :Author:    Lele Gaifax <lele@metapensiero.it>
 # :License:   GNU General Public License version 3 or later
-# :Copyright: © 2018, 2020 Lele Gaifax
+# :Copyright: © 2018, 2020, 2022 Lele Gaifax
 #
 
 from metapensiero.sqlalchemy.proxy.json import JSON
@@ -208,8 +208,7 @@ def test_active_players(guest_user, session, club_scr):
     assert 1 < result['count'] < session.query(Player).count()
 
 
-def test_player_by_nickname(guest_user, player_lele, tourney_rated_no_turns_odd,
-                            tourney_closed):
+def test_player_by_nickname(guest_user, player_lele, tourney_closed):
     response = guest_user.get_route('players',
                                     _query={'filter_by_nickname': player_lele.nickname})
     result = response.json
@@ -222,6 +221,26 @@ def test_player_by_nickname(guest_user, player_lele, tourney_rated_no_turns_odd,
     assert player['LastPlayed'] == tourney_closed.date.isoformat()
 
 
+def test_player_by_name(lele_user):
+    filters = '[{"property":"firstname","value":"rob","operator":"~"}]'
+    response = lele_user.get_route('players', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 6
+    assert {player['firstname'] for player in result['root']} == {
+        'Roberto', 'Cheryl', 'Darren', 'Robert', 'Daniel'}
+
+    filters = ('[{"property":"firstname","value":"rob","operator":"~"}'
+               ',{"property":"lastname","value":"blo","operator":"~"}]')
+    response = lele_user.get_route('players', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 1
+    assert result['root'][0]['firstname'] == 'Roberto'
+
+
 def test_owned_player(guest_user, player_fata, user_lele):
     response = guest_user.get_route('players',
                                     _query={'filter_by_firstname': player_fata.firstname,
@@ -385,6 +404,40 @@ def test_users(guest_user, lele_user):
     assert result['root'][1]['Language'] == 'Inglese (Stati Uniti)'
 
 
+def test_users_filter(lele_user):
+    filters = '[{"property":"lastname","value":"lele","operator":"~"}]'
+    response = lele_user.get_route('users', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 1
+    assert result['root'][0]['lastname'] == 'Gaifax'
+
+    filters = '[{"property":"email","value":"lele","operator":"~"}]'
+    response = lele_user.get_route('users', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 1
+    assert result['root'][0]['lastname'] == 'Gaifax'
+
+    filters = '[{"property":"firstname","value":"metap","operator":"~"}]'
+    response = lele_user.get_route('users', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 1
+    assert result['root'][0]['lastname'] == 'Gaifax'
+
+    filters = ('[{"property":"firstname","value":"lele","operator":"~"}'
+               ',{"property":"lastname","value":"metap","operator":"~"}]')
+    response = lele_user.get_route('users', _query={'filter': filters})
+    result = response.json
+    assert result['success'] is True
+    assert result['message'] == "Ok"
+    assert result['count'] == 0
+
+
 def test_users_metadata(lele_user):
     response = lele_user.get_route('users', _query={'metadata': 'metadata'})
     result = response.json
